def get_config( expanded=False ):
    """
    Get configuration values as strings which can be logged, stored or evaluated.

    Parameters
       expanded (bool=False) - If True, return the configuration values with all path values expanded using expanduser and abspath from the os.path module.

    Returns
       list[str] - list of configuration strings
    """

    from .. import config as _config
    if expanded :
        valsObj = _config
    else:
        valsObj = _config._config_defaults

    return list( map( lambda name: f'{name} = {repr(getattr(valsObj,name))}', _config.__defaults ) )



# Copyright 2020 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
"""
this module will be included in the api
"""

def measures_update(path=None, version=None, force=False, logger=None):
    """
    Retrieve IERS data used for measures calculations from ASTRON FTP server
    
    Original data source is here: https://www.iers.org/IERS/EN/DataProducts/data.html
    
    Parameters
       - path (str=None) - Folder path to place updated measures data. Default None places it in package installation directory
       - version (str=None) - Version of measures data to retrieve (usually in the form of yyyymmdd-160001.ztar, see measures_available()). Default None retrieves the latest
       - force (bool=False) - If True, always re-download the measures data. Default False will not download measures data if already updated today unless version parameter is specified and different from what was last downloaded.
       - logger (casatools.logsink=None) - Instance of the casalogger to use for writing messages. Default None writes messages to the terminal
        
    Returns
       None
    
    """
    from ftplib import FTP
    import os
    from datetime import datetime
    import pkg_resources
    import sys
    
    if path is None: path = pkg_resources.resource_filename('casaconfig', '__data__/')
    path = os.path.expanduser(path)
    if not os.path.exists(path): os.mkdir(path)
    current = None
    updated = None

    # if measures are already preset, get their version
    if os.path.exists(os.path.join(path, 'geodetic/readme.txt')):
        try:
            with open(os.path.join(path,'geodetic/readme.txt'), 'r') as fid:
                readme = fid.readlines()
            current = readme[1].split(':')[-1].strip()
            updated = readme[2].split(':')[-1].strip()
        except:
            pass

    # don't re-download the same data
    if not force:
        if ((version is not None) and (version == current)) or ((version is None) and (updated == datetime.today().strftime('%Y-%m-%d'))):
            print('casaconfig current measures detected in %s, using version %s' % (path, current), file = sys.stderr )
            if logger is not None: logger.post('casaconfig current measures detected in %s, using version %s' % (path, current), 'INFO')
            return

    # path must be writable with execute bit set
    if (not os.access(path, os.W_OK | os.X_OK)) :
        print('No permission to write to the measures path, cannot update : %s' % path, file = sys.stderr )
        if logger is not None: logger.post('No permission to write to the measures path, cannot update : %s' % path, 'ERROR')
        return

    print('casaconfig connecting to ftp.astron.nl ...', file = sys.stderr )
    if logger is not None: logger.post('casconfig connecting to ftp.astron.nl ...', 'INFO')

    ftp = FTP('ftp.astron.nl')
    rc = ftp.login()
    rc = ftp.cwd('outgoing/Measures')
    files = sorted([ff for ff in ftp.nlst() if (len(ff) > 0) and (not ff.endswith('.dat')) and (ftp.size(ff) > 0)])

    # target filename to download
    target = files[-1] if version is None else version
    if target not in files:
        if logger is not None: logger.post('casaconfig cant find specified version %s' % target, 'ERROR')
        else: print('##### ERROR: cant find specified version %s #####' % target)
        return
    
    with open(os.path.join(path,'measures.ztar'), 'wb') as fid:
        print('casaconfig downloading %s from ASTRON server to %s ...' % (target, path), file = sys.stderr )
        if logger is not None: logger.post('casaconfig downloading %s from ASTRON server to %s ...' % (target, path), 'INFO')
        ftp.retrbinary('RETR ' + target, fid.write)
    
    os.system("tar -zxf %s -C %s" % (os.path.join(path,'measures.ztar'), path))
    os.system("rm %s" % os.path.join(path, 'measures.ztar'))
    os.system("rm -fr %s/*.old" % os.path.join(path, 'geodetic'))
    with open(os.path.join(path,'geodetic/readme.txt'), 'w') as fid:
       fid.write("# measures data populated by casaconfig\nversion : %s\ndate : %s" % (target, datetime.today().strftime('%Y-%m-%d')))

    return



# Copyright 2022 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

## get an argparse.ArgumentParser suitable for parsing the configurable
## parameters known to casa, help is turned off here but can be turned on in the returned parser
## this parser includes the arguments used by casaconfig, additional arguments can be added

def get_argparser(add_help=False):
    import argparse as __argparse

    ## look for arguments affecting the configuration
    parser = __argparse.ArgumentParser(add_help=add_help)
    parser.add_argument( "--noconfig", dest='noconfig', action='store_const', const=True, default=False,
                         help='do not load user configuration file' )
    parser.add_argument( "--configfile",dest='configfile', default='~/.casa/config.py',
                         help='location of the user configuration file')
    return parser



# Copyright 2020 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
"""
this module will be included in the api
"""


def measures_available():
    """
    List available measures versions on ASTRON FTP server

    Parameters
       None
    
    Returns
       list - version names returned as list of strings

    """
    from ftplib import FTP
    
    ftp = FTP('ftp.astron.nl')
    rc = ftp.login()
    rc = ftp.cwd('outgoing/Measures')
    files = ftp.nlst()
    ftp.quit()
    #files = [ff.replace('WSRT_Measures','').replace('.ztar','').replace('_','') for ff in files]
    files = [ff for ff in files if (len(ff) > 0) and (not ff.endswith('.dat'))]
    return files



# search path for measurement sets and images to load
datapath = [ ]

# location of geodetic and ephemera data
measurespath = "~/.casa/measures"

# location of the cachedir
cachedir = '~/.casa'

# location of the optional user's startup.py
startupfile = '~/.casa/startup.py'

# automatically populate the measurespath[0] location if not already done
populate_data = True

# automatically update measures data if not current (measurespath must be user-writable)
measures_update = True

# log file path/name
logfile='casa-%s.log' % _time.strftime("%Y%m%d-%H%M%S", _time.gmtime())

# do not create a log file when True, If True, then any logfile value is ignored and there is no log file
nologfile = False

# print log output to terminal when True (in addition to any logfile and CASA logger)
log2term = False

# do not start the CASA logger when True
nologger = False

# avoid starting GUI tools when True. If True then the CASA logger is not started even if nologger is False
nogui = False

# the IPython prompt color scheme. Must be one of "Neutral", "NoColor", "Linux" or "LightBG", default "Neutral"
colors = "Neutral"

# startup without a graphical backend if True
agg = False

# attempt to load the pipeline modules and set other options appropriate for pipeline use if True
pipeline = False

# create and use an IPython log in the current directory if True
iplog = False

# allow anonymous usage reporting
telemetry_enabled = True

# location to place telemetry data prior to reporting
telemetry_log_directory = '~/.casa/telemetry'

# maximum size of telemetry recording
telemetry_log_limit = 20480

# telemetry recording size that triggers a report
telemetry_log_size_interval = 60

# telemetry recording report frequency
telemetry_submit_interval = 604800

# allow anonymous crash reporting
crashreporter_enabled = True

# include the user's local site-packages in the python path if True. May conflict with CASA modules
user_site = False
